@@ -1,4 +1,4 @@
-/* xscreensaver, Copyright (c) 2006-2012 Jamie Zawinski <jwz@jwz.org>
+/* xscreensaver, Copyright (c) 2006-2013 Jamie Zawinski <jwz@jwz.org>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -69,9 +69,9 @@
 
 }
 
-- (id)initWithXMLFile: (NSString *) xml_file
-              options: (const XrmOptionDescRec *) opts
-           controller: (NSUserDefaultsController *) prefs
-             defaults: (NSDictionary *) defs;
+- (id)initWithXML: (NSData *) xml_data
+          options: (const XrmOptionDescRec *) opts
+       controller: (NSUserDefaultsController *) prefs
+         defaults: (NSDictionary *) defs;
 
 @end
