@@ -25,6 +25,19 @@
 #include "xlock.h"
 #endif
 
+#ifdef HAVE_COCOA
+# include "jwxyz.h"
+#else
+# include <X11/Xlib.h>
+# include <GL/gl.h>
+# include <GL/glu.h>
+#endif
+
+#ifdef HAVE_JWZGLES
+# include "jwzgles.h"
+#endif /* HAVE_JWZGLES */
+
+#include "sphere.h"
 #include "gltrackball.h"
 
 #define DEF_SHADOWS  "True"
@@ -148,6 +161,7 @@ static const GLfloat ground[4] = {0.0, 1.0, 0.0, -0.00001};
 /* simple filled sphere */
 static Bool mySphere(float radius) 
 {
+#if 0
   GLUquadricObj *quadObj;
 
   if((quadObj = gluNewQuadric()) == 0)
@@ -155,13 +169,20 @@ static Bool mySphere(float radius)
   gluQuadricDrawStyle(quadObj, (GLenum) GLU_FILL);
   gluSphere(quadObj, radius, 16, 16);
   gluDeleteQuadric(quadObj);
-
+#else
+    glPushMatrix();
+    glScalef (radius, radius, radius);
+    glRotatef (90, 1, 0, 0);
+    unit_sphere (16, 16, False);
+    glPopMatrix();
+#endif
   return True;
 }
 
 /* caged sphere */
 static Bool mySphere2(float radius) 
 {
+#if 0
   GLUquadricObj *quadObj;
 
   if((quadObj = gluNewQuadric()) == 0)
@@ -169,6 +190,13 @@ static Bool mySphere2(float radius)
   gluQuadricDrawStyle(quadObj, (GLenum) GLU_LINE);/*GLU_SILHOUETTE);*/
   gluSphere(quadObj, radius, 16, 8);
   gluDeleteQuadric(quadObj);
+#else
+    glPushMatrix();
+    glScalef (radius, radius, radius);
+    glRotatef (90, 1, 0, 0);
+    unit_sphere (16, 8, True);
+    glPopMatrix();
+#endif
 
   return True;
 }
@@ -180,7 +208,7 @@ static Bool myCone2(float radius)
 }
 
 /* draw an ant */
-static Bool draw_antinspect_ant(antinspectstruct * mp, 
+static Bool draw_antinspect_ant(ModeInfo *mi, antinspectstruct * mp, 
                                 const float *Material, int mono,
                                 Bool (*sphere)(float), Bool (*cone)(float)) 
 {
@@ -235,6 +263,7 @@ static Bool draw_antinspect_ant(antinspectstruct * mp,
   glVertex3f(0.00, 0.30, 0.00);
   glColor3fv(MaterialGray);
   glVertex3f(0.40, 0.70, 0.40);
+  mi->polygon_count++;
   if (mono)
     glColor3fv(MaterialGray5);
   else
@@ -242,6 +271,7 @@ static Bool draw_antinspect_ant(antinspectstruct * mp,
   glVertex3f(0.00, 0.30, 0.00);
   glColor3fv(MaterialGray);
   glVertex3f(0.40, 0.70, -0.40);
+  mi->polygon_count++;
   glEnd();
   glBegin(GL_POINTS);
   if (mono)
@@ -249,7 +279,9 @@ static Bool draw_antinspect_ant(antinspectstruct * mp,
   else
     glColor3fv(Material);
   glVertex3f(0.40, 0.70, 0.40);
+  mi->polygon_count++;
   glVertex3f(0.40, 0.70, -0.40);
+  mi->polygon_count++;
   glEnd();
   
   /* LEFT-FRONT ARM */
@@ -260,8 +292,10 @@ static Bool draw_antinspect_ant(antinspectstruct * mp,
     glColor3fv(Material);
   glVertex3f(0.00, 0.05, 0.18);
   glVertex3f(0.35 + 0.05 * cos1, 0.15, 0.25);
+  mi->polygon_count++;
   glColor3fv(MaterialGray);
   glVertex3f(-0.20 + 0.05 * cos1, 0.25 + 0.1 * sin1, 0.45);
+  mi->polygon_count++;
   glEnd();
   
   /* LEFT-CENTER ARM */
@@ -272,9 +306,12 @@ static Bool draw_antinspect_ant(antinspectstruct * mp,
     glColor3fv(Material);
   glVertex3f(0.00, 0.00, 0.18);
   glVertex3f(0.35 + 0.05 * cos2, 0.00, 0.25);
+  mi->polygon_count++;
   glColor3fv(MaterialGray);
   glVertex3f(-0.20 + 0.05 * cos2, 0.00 + 0.1 * sin2, 0.45);
+  mi->polygon_count++;
   glEnd();
+  mi->polygon_count++;
   
   /* LEFT-BACK ARM */
   glBegin(GL_LINE_STRIP);
@@ -284,8 +321,10 @@ static Bool draw_antinspect_ant(antinspectstruct * mp,
     glColor3fv(Material);
   glVertex3f(0.00, -0.05, 0.18);
   glVertex3f(0.35 + 0.05 * cos3, -0.15, 0.25);
+  mi->polygon_count++;
   glColor3fv(MaterialGray);
   glVertex3f(-0.20 + 0.05 * cos3, -0.25 + 0.1 * sin3, 0.45);
+  mi->polygon_count++;
   glEnd();
   
   /* RIGHT-FRONT ARM */
@@ -296,8 +335,10 @@ static Bool draw_antinspect_ant(antinspectstruct * mp,
     glColor3fv(Material);
   glVertex3f(0.00, 0.05, -0.18);
   glVertex3f(0.35 - 0.05 * sin1, 0.15, -0.25);
+  mi->polygon_count++;
   glColor3fv(MaterialGray);
   glVertex3f(-0.20 - 0.05 * sin1, 0.25 + 0.1 * cos1, -0.45);
+  mi->polygon_count++;
   glEnd();
   
   /* RIGHT-CENTER ARM */
@@ -308,8 +349,10 @@ static Bool draw_antinspect_ant(antinspectstruct * mp,
     glColor3fv(Material);
   glVertex3f(0.00, 0.00, -0.18);
   glVertex3f(0.35 - 0.05 * sin2, 0.00, -0.25);
+  mi->polygon_count++;
   glColor3fv(MaterialGray);
   glVertex3f(-0.20 - 0.05 * sin2, 0.00 + 0.1 * cos2, -0.45);
+  mi->polygon_count++;
   glEnd();
   
   /* RIGHT-BACK ARM */
@@ -320,8 +363,10 @@ static Bool draw_antinspect_ant(antinspectstruct * mp,
     glColor3fv(Material);
   glVertex3f(0.00, -0.05, -0.18);
   glVertex3f(0.35 - 0.05 * sin3, -0.15, -0.25);
+  mi->polygon_count++;
   glColor3fv(MaterialGray);
   glVertex3f(-0.20 - 0.05 * sin3, -0.25 + 0.1 * cos3, -0.45);
+  mi->polygon_count++;
   glEnd();
     
   glEnable(GL_LIGHTING);
@@ -377,6 +422,7 @@ static Bool draw_antinspect_strip(ModeInfo * mi)
   glVertex3f(0.0, 0.0, -1.0);
   glVertex3f(-sqrt(3.0)/2.0, 0.0, 0.5);
   glVertex3f(sqrt(3.0)/2.0, 0.0, 0.5);
+  mi->polygon_count++;
   glEnd();
 
   /* rotate */
@@ -386,6 +432,7 @@ static Bool draw_antinspect_strip(ModeInfo * mi)
     glVertex3f(0.0, 0.0, 1.0 + 3.0);
     glVertex3f(sqrt(3.0)/2.0, 0.0, -0.5 + 3.0);
     glVertex3f(-sqrt(3.0)/2.0, 0.0, -0.5 + 3.0);
+    mi->polygon_count++;
     glEnd();
   }
 
@@ -429,7 +476,7 @@ static Bool draw_antinspect_strip(ModeInfo * mi)
 
       if(antposition[i] > 360.0)
 	antposition[i] = 0.0;
-      draw_antinspect_ant(mp, MaterialShadow, mono, mySphere2, myCone2);
+      draw_antinspect_ant(mi, mp, MaterialShadow, mono, mySphere2, myCone2);
 
       glDisable(GL_BLEND);
       glDisable(GL_LIGHTING);
@@ -471,7 +518,7 @@ static Bool draw_antinspect_strip(ModeInfo * mi)
     if(antposition[i] > 360.0)
       antposition[i] = 0.0;
     glEnable(GL_BLEND);
-    draw_antinspect_ant(mp, antmaterial[i], mono, mySphere2, myCone2);
+    draw_antinspect_ant(mi, mp, antmaterial[i], mono, mySphere2, myCone2);
     glDisable(GL_BLEND);
     glPopMatrix();
 
@@ -518,8 +565,6 @@ ENTRYPOINT void reshape_antinspect(ModeInfo * mi, int width, int height)
 static void pinit(void) 
 {
   glClearDepth(1.0);
-  glClearColor(0.0, 0.0, 0.0, 1.0);
-  
   glLightfv(GL_LIGHT0, GL_AMBIENT, ambient);
   glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuse);
   glLightfv(GL_LIGHT0, GL_POSITION, position0);
@@ -570,7 +615,9 @@ ENTRYPOINT Bool antinspect_handle_event (ModeInfo *mi, XEvent *event)
   }
   else if (event->xany.type == ButtonPress &&
            (event->xbutton.button == Button4 ||
-            event->xbutton.button == Button5))
+            event->xbutton.button == Button5 ||
+            event->xbutton.button == Button6 ||
+            event->xbutton.button == Button7))
     {
       gltrackball_mousewheel (mp->trackball, event->xbutton.button, 5,
                               !event->xbutton.state);
@@ -632,6 +679,9 @@ ENTRYPOINT void draw_antinspect(ModeInfo * mi)
   glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
 
   glPushMatrix();
+  glRotatef(current_device_rotation(), 0, 0, 1);
+
+  mi->polygon_count = 0;
 
   /* position camera --- this works well, we can peer inside 
      the antbubble */
